#! /bin/bash
Current_DIR=$(cd `dirname $0`; pwd)
valuesFile=$Current_DIR/values.yaml

getValueFromValues(){
  fileName=$2
  [[ -z $fileName ]] && fileName=$valuesFile
  index=$(sed -n '/'"$1":'/=' $fileName)
  if [ -z $index ];then
    echo ""
    return $?
  fi
  lineStr=$(cat $fileName | awk 'NR=='"$index"'')
  echo ${lineStr: $(expr length $1)+2}
  return $?
}

replaceFileContent() {
  if [[ `echo $operatorSystem | grep -ciw "Darwin"` = 1 ]]
  then
    # Mac
    sed -i '' $@
  else
    # Linux
    sed -i "$@"
  fi
}

# istioctl 命令参数
function command_args() {
  deploy_registry=$(getValueFromValues deploy_registry)
  deploy_istio_enable_egress=$(getValueFromValues deploy_istio_enable_egress)
  deploy_istio_enable_kiali_tracing=$(getValueFromValues deploy_istio_enable_kiali_tracing)
  deploy_imanager_namespace=$(getValueFromValues deploy_namespace)
  deploy_image_pull_secret=$(getValueFromValues deploy_image_pull_secret)
  deploy_image_namespace=$(getValueFromValues deploy_image_namespace)

  [[ -z $deploy_registry ]] && deploy_registry=registry.cn-beijing.aliyuncs.com
  [[ -z $deploy_istio_enable_egress ]] && deploy_istio_enable_egress=true
  [[ -z $deploy_istio_enable_kiali_tracing ]] && deploy_istio_enable_kiali_tracing=true
  [[ -z $deploy_imanager_namespace ]] && deploy_imanager_namespace=supermap
  [[ -z $deploy_image_pull_secret ]] && deploy_image_pull_secret=none
  [[ -z $deploy_image_pull_policy ]] && deploy_image_pull_policy=IfNotPresent
  [[ -z $deploy_image_namespace ]] && deploy_image_namespace=supermap

  deploy_registry=${deploy_registry}/${deploy_image_namespace}

  cp -f $Current_DIR/istio-operator.yaml.tmpl $Current_DIR/istio-operator.yaml
  replaceFileContent 's|${REGISTRY_URL}|'"$deploy_registry"'|' $Current_DIR/istio-operator.yaml
  replaceFileContent 's|${ISTIO_ENABLE_EGRESS}|'"$deploy_istio_enable_egress"'|' $Current_DIR/istio-operator.yaml
  replaceFileContent 's|${ISTIO_ENABLE_KIALI_TRACING}|'"$deploy_istio_enable_kiali_tracing"'|' $Current_DIR/istio-operator.yaml
  replaceFileContent 's|${IMAGE_PULL_SECRET}|'"$deploy_image_pull_secret"'|' $Current_DIR/istio-operator.yaml
  replaceFileContent 's|${IMAGE_PULL_POLICY}|'"$deploy_image_pull_policy"'|' $Current_DIR/istio-operator.yaml
  replaceFileContent 's|${IMANAGER_NAMESPACE}|'"$deploy_imanager_namespace"'|' $Current_DIR/istio-operator.yaml
}

# 开启 istio
function open() {
  command_args
  $Current_DIR/istioctl manifest apply -f $Current_DIR/istio-operator.yaml --readiness-timeout 10m
}

# 关闭 istio
function close() {
  command_args
  $Current_DIR/istioctl manifest generate -f $Current_DIR/istio-operator.yaml | kubectl delete --ignore-not-found=true -f -;
  kubectl delete ns istio-system;
}

istio_enable=$1

if [[ "$istio_enable" == "open" ]]; then
  open
else
  close
fi